#!/usr/bin/php
<?php
// https://jira.op5.com/browse/MON-6154 workaround

class Base
{
	protected $pdo = NULL;
	public $report = array();

	public function __construct()
	{
		set_error_handler(array($this, 'exception_error_handler'));
	}

	public function exception_error_handler($errno, $errstr, $errfile, $errline)
	{
		throw new ErrorException($errstr, $errno, 0, $errfile, $errline);
	}

	public function exception_parse($e)
	{
		$ret = array();
		do
		{
			$trace = $e->getTrace();
			$func = $trace[0]['function'];
			$ret[] = array(
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'func' => $func,
				'msg' => $e->getMessage()
			);
		}
		while($e = $e->getPrevious());

		return $ret;
	}

	public function exception_die($e)
	{
		$trace = $this->exception_parse($e);
		foreach(array_reverse($trace) as $n => $a)
		{
			printf(
				"#%d %s@%s %s(): %s\n",
				$n, $a['file'], $a['line'], $a['func'], $a['msg']
			);
		}
		exit();
	}

	public function msg($msg)
	{
		echo date('[H:m:s] ')."$msg\n";
	}

	public function db_init($db_settings=array())
	{
		if(!is_array($db_settings))
			throw new Exception('PARAM_NOT_ARRAY');

		$names = array('host', 'name', 'pass', 'user', 'type');
		foreach($names as $name)
			if(!isset($db_settings[$name]))
				throw new Exception('MISSING_SETTING:' . $name);

		$t = $db_settings['type'];
		$h = $db_settings['host'];
		$n = $db_settings['name'];

		try {
			$this->pdo = new PDO(
				"{$t}:host={$h};dbname=${n}",
				$db_settings['user'],
				$db_settings['pass']
			);
		} catch(PDOException $e) {
			throw new Exception('PDO_CONNECT_ERROR', 0, $e);
		}

		if(!$this->pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION))
			throw new Exception('PDO_SET_ATTRIBUTE_ERROR');

		return true;
	}

	protected function db_sth_prep_exec($query, $params=array())
	{
		if(!is_object($this->pdo))
			throw new Exception('PDO_NOT_INITIALIZED');

		// prepare query
		try {
			$sth = $this->pdo->prepare($query);
		} catch(PDOException $e) {
			throw new Exception('PDO_PREPARE', 0, $e);
		}

		// execute query
		try {
			$sth->execute($params);
		} catch(PDOException $e) {
			throw new Exception('PDO_EXECUTE', 0, $e);
		}

		return $sth;
	}

	public function get_db_months()
	{
		$query = '
			SELECT
				saved_reports_options.report_id AS id,
				saved_reports_options.value AS value,
				saved_reports.report_name AS name
			FROM
				saved_reports_options
			LEFT JOIN saved_reports
				ON saved_reports_options.report_id=saved_reports.id
			WHERE
				saved_reports_options.name = "months"
				AND
				saved_reports.type = "sla"
		';

		// prepare and exec query
		try {
			$sth = $this->db_sth_prep_exec($query);
		} catch(PDOException $e) {
			throw new Exception('ERROR_INSERTING_ENTRY', $e);
		}

		foreach($sth->fetchAll(PDO::FETCH_ASSOC) as $r)
		{
			if(empty($r['value']))
				continue;
			if(empty($r['name']))
				continue;

			$id = $r['id'];
			$this->report[$id] = array(
				'name' => $r['name'],
				'raw' => $r['value']
			);
		}
	}


	public function determine_updates()
	{
		foreach($this->report as $id => $data)
		{
			$name = $data['name'];
			$months_raw = $data['raw'];

			if(!$months = unserialize($months_raw))
				throw new Exception("Unable to unserialize id '$id' (data:$months_raw)");

			$months_unique = array_unique($months);
			$c = count($months_unique);
			if($c > 1)
			{
				$this->msg("Ignoring report '$name' (id '$id') due to multiple SLA values.");
				unset($this->report[$id]);
				continue;
			}

			foreach($months_unique as $sla)
				break;

			$this->report[$id]['sla'] = $sla;
		}
	}

	public function update_db_months($id)
	{
		if(!isset($id))
			throw new Exception('MISSING_PARAM:id');

		$sla = $this->report[$id]['sla'];
		$name = $this->report[$id]['name'];

		$this->msg("Report '$name' (id: $id): SLA value: $sla");

		$value = array();
		for ($n=1; $n<=12; $n++)
			$value[$n] = $sla;

		$query = "
			UPDATE saved_reports_options
			SET value = :value
			WHERE
				report_id = :id
				AND
				name = 'months'
		";

		$params = array(
			'value' => serialize($value),
			'id' => $id
		);

		// prepare and exec query
		try {
			$sth = $this->db_sth_prep_exec($query, $params);
		} catch(PDOException $e) {
			throw new Exception('ERROR_INSERTING_ENTRY', $e);
		}
	}
}

function syntax($code=0)
{
	echo "Usage: op5 mt saved-reports-prop-sla-months <--run>\n";
	exit($code);
}


$o = new Base;

if(!is_object($o))
	die("MAIN_OBJECT_NOT_CREATED\n");

if(!isset($argv[1]))
	syntax(1);
if($argv[1] === '-h' || $argv[1] === '--help')
	syntax(0);
if($argv[1] !== '--run')
	syntax(1);


$db_settings = array(
	'host' => 'localhost',
	'user' => 'root',
	'pass' => '',
	'name' => 'merlin',
	'type' => 'mysql',
	'timeout' => 10
);

try {
	$o->db_init($db_settings);

	$o->msg('Fetching SLA values from saved_reports_options db table...');
	$o->get_db_months();

	$o->msg('Determining which entries to update...');
	$o->determine_updates();

	echo "\n";
	foreach($o->report as $id => $data)
		$o->update_db_months($id);

	echo "\n";
	$o->msg('All done.');

} catch(Exception $e) {
	$o->exception_die($e);
}


?>
